Imports System.ServiceProcess

Public Class Service1
    Inherits System.ServiceProcess.ServiceBase

    Public lProps As listenerProperties
    Public WithEvents serialConn As Rs232
    Public WithEvents udpConn As Nsock
    Public BufStr As String
    Public oLogFile As System.IO.TextWriter
    Public objEventLog As New EventLog()

#Region " Component Designer generated code "

    Public Sub New()
        MyBase.New()

        ' This call is required by the Component Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call

    End Sub

    'UserService overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    ' The main entry point for the process
    <MTAThread()> _
    Shared Sub Main()
        Dim ServicesToRun() As System.ServiceProcess.ServiceBase

        ' More than one NT Service may run within the same process. To add
        ' another service to this process, change the following line to
        ' create a second service object. For example,
        '
        '   ServicesToRun = New System.ServiceProcess.ServiceBase () {New Service1, New MySecondUserService}
        '
        ServicesToRun = New System.ServiceProcess.ServiceBase() {New Service1()}

        System.ServiceProcess.ServiceBase.Run(ServicesToRun)
    End Sub

    'Required by the Component Designer
    Private components As System.ComponentModel.IContainer

    ' NOTE: The following procedure is required by the Component Designer
    ' It can be modified using the Component Designer.  
    ' Do not modify it using the code editor.
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        components = New System.ComponentModel.Container()
        Me.ServiceName = "Service1"
    End Sub

#End Region

    Protected Overrides Sub OnStart(ByVal args() As String)
        Dim i As Int32
        lProps = New listenerProperties()

        'Get/Set the stored setting from teh registry
        lProps.Broadcast = GetSetting("WhozzCalling Listener Service", "settings", "Broadcast", True)
        lProps.BroadcastNet = GetSetting("WhozzCalling Listener Service", "settings", "BroadcastNet", "255.255.255.255")
        lProps.BroadcastPort = GetSetting("WhozzCalling Listener Service", "settings", "BroadcastPort", "6699")
        lProps.ComPort = GetSetting("WhozzCalling Listener Service", "settings", "ComPort", 1)
        lProps.KeepLog = GetSetting("WhozzCalling Listener Service", "settings", "KeepLog", False)
        lProps.LogFile = GetSetting("WhozzCalling Listener Service", "settings", "LogFile", App_Path() & "whozzcalling_listener.txt")
        lProps.ToggleString = GetSetting("WhozzCalling Listener Service", "settings", "ToggleString")

        'Parse arguments passed to the service through the Services Panel
        parseArgs(args)

        'Create a log entry showing the current version and settings
        Try
            If Not objEventLog.SourceExists("WhozzCallingService") Then
                objEventLog.CreateEventSource("WhozzCallingService", "WhozzCalling Log")
            End If
            Dim startlogstring As New String("***WhozzCalling LIstener Service Started***" & ControlChars.NewLine _
            & "Version " & lProps.Version & ControlChars.NewLine _
            & "Logging: " & lProps.KeepLog & ControlChars.NewLine _
            & "LogFile: " & lProps.LogFile & ControlChars.NewLine _
            & "Broadcast: " & lProps.Broadcast & ControlChars.NewLine _
            & "Broadcast: " & lProps.BroadcastNet & ":" & lProps.BroadcastPort & ControlChars.NewLine _
            & "Com Port: " & lProps.ComPort & ControlChars.NewLine _
            & "ToggleString: " & lProps.ToggleString & ControlChars.NewLine)

            objEventLog.Source = "WhozzCallingService"
            objEventLog.WriteEntry(startlogstring)
        Catch Exa As Exception
        End Try

        'Open the log file
        If lProps.KeepLog Then
            oLogFile = System.IO.File.AppendText(lProps.LogFile)
            oLogFile.WriteLine("Logfile started: " & Now.ToString())
        End If

        'Start our connection to the serial port
        serialConn = New Rs232()

        Try
            'Set up port
            With serialConn
                .Port = lProps.ComPort
                .BaudRate = 9600
                .DataBit = 8
                .StopBit = Rs232.DataStopBit.StopBit_1
                .Parity = Rs232.DataParity.Parity_None
                .Timeout = 1000
            End With

            If serialConn.IsOpen() Then
                serialConn.Close()
            End If
            serialConn.Open()

            serialConn.Dtr = True
            serialConn.Rts = True
            serialConn.EnableEvents()
        Catch Ex As Exception
            'Could not open the com port...create a log entry to mark the event
            Try
                If Not objEventLog.SourceExists("WhozzCallingService") Then
                    objEventLog.CreateEventSource("WhozzCallingService", "WhozzCalling Log")
                End If
                objEventLog.Source = "WhozzCallingService"
                objEventLog.WriteEntry("ERROR: Could not initialize COM port")
            Catch Exa As Exception
            End Try
        End Try

        'If a toggle string was set, go through each character of the string, and send it to the unit
        If lProps.ToggleString.Length > 0 Then
            For i = 0 To lProps.ToggleString.Length - 1
                serialConn.Write(lProps.ToggleString.Substring(i, 1))
                System.Threading.Thread.Sleep(50) 'give the unit time to recognize the received signal
            Next
        End If

        'Open the UDP broadcast port
        udpConn = New Nsock()
    End Sub

    Protected Overrides Sub OnStop()
        'Close the serial port connection
        If serialConn.IsOpen() Then
            serialConn.Close()
        End If

        'Close the logfile
        Try
            oLogFile.Close()
        Catch ex As Exception
        End Try

        'Save the current settings to the registry
        SaveSetting("WhozzCalling Listener Service", "settings", "KeepLog", lProps.KeepLog)
        SaveSetting("WhozzCalling Listener Service", "settings", "LogFile", lProps.LogFile)
        SaveSetting("WhozzCalling Listener Service", "settings", "Broadcast", lProps.Broadcast)
        SaveSetting("WhozzCalling Listener Service", "settings", "BroadcastNet", lProps.BroadcastNet)
        SaveSetting("WhozzCalling Listener Service", "settings", "BroadcastPort", lProps.BroadcastPort)
        SaveSetting("WhozzCalling Listener Service", "settings", "ComPort", lProps.ComPort)
        SaveSetting("WhozzCalling Listener Service", "settings", "ToggleString", lProps.ToggleString)
    End Sub

    Public Sub parseArgs(ByVal args() As String)
        'Simply parses through the arguments passed to the service, and overwrites the current settings w/ new specified ones
        Dim arg As String
        Dim argc, i As Int32

        argc = args.Length

        For i = 0 To argc - 1
            Select Case args(i)
                Case "-p"
                    lProps.ComPort = args(i + 1)
                Case "-l"
                    lProps.KeepLog = True
                    lProps.LogFile = args(i + 1)
                Case "-b"
                    lProps.BroadcastPort = args(i + 1)
                Case "-k"
                    lProps.BroadcastNet = args(i + 1)
                Case "-n"
                    lProps.Broadcast = False
                Case "-t"
                    lProps.ToggleString = args(i + 1)
                Case "-x"
                    lProps.KeepLog = False
            End Select
        Next
    End Sub

    Public Function App_Path() As String
        'Helper function - finds the path to the executable - used for default logfile location
        Return System.AppDomain.CurrentDomain.BaseDirectory()
    End Function

    Private Sub serialConn_CommEvent(ByVal source As Rs232, ByVal Mask As Rs232.EventMasks) Handles serialConn.CommEvent
        Dim iPnt As Int32, sBuf As String, Buffer() As Byte

        If (Mask And Rs232.EventMasks.RxChar) > 0 Then
            Buffer = source.InputStream
            For iPnt = 0 To Buffer.Length - 1
                If Buffer(iPnt).ToString("X").Equals("A") Then ' recieved 0x10 (Cr)

                    ' send data over Nsock
                    udpConn.SendText(BufStr, lProps.BroadcastNet, lProps.BroadcastPort)

                    ' write data to the log file
                    If lProps.KeepLog Then
                        oLogFile.WriteLine(BufStr)
                    End If

                    ' reset BufStr for next line
                    BufStr = ""
                ElseIf Buffer(iPnt).ToString("X").Equals("D") Then
                    ' do nothing - this strips out 0x13
                Else
                    BufStr += Chr(Buffer(iPnt))
                End If
            Next
        End If
    End Sub

End Class
